/*******************************************************************************
* DISCLAIMER
* This software is supplied by Renesas Electronics Corporation and is only
* intended for use with Renesas products. No other uses are authorized. This
* software is owned by Renesas Electronics Corporation and is protected under
* all applicable laws, including copyright laws.
* THIS SOFTWARE IS PROVIDED "AS IS" AND RENESAS MAKES NO WARRANTIES REGARDING
* THIS SOFTWARE, WHETHER EXPRESS, IMPLIED OR STATUTORY, INCLUDING BUT NOT
* LIMITED TO WARRANTIES OF MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE
* AND NON-INFRINGEMENT. ALL SUCH WARRANTIES ARE EXPRESSLY DISCLAIMED.
* TO THE MAXIMUM EXTENT PERMITTED NOT PROHIBITED BY LAW, NEITHER RENESAS
* ELECTRONICS CORPORATION NOR ANY OF ITS AFFILIATED COMPANIES SHALL BE LIABLE
* FOR ANY DIRECT, INDIRECT, SPECIAL, INCIDENTAL OR CONSEQUENTIAL DAMAGES FOR
* ANY REASON RELATED TO THIS SOFTWARE, EVEN IF RENESAS OR ITS AFFILIATES HAVE
* BEEN ADVISED OF THE POSSIBILITY OF SUCH DAMAGES.
* Renesas reserves the right, without notice, to make changes to this software
* and to discontinue the availability of this software. By using this software,
* you agree to the additional terms and conditions found by accessing the
* following link:
* http://www.renesas.com/disclaimer
*
* Copyright (C) 2013 Renesas Electronics Corporation. All rights reserved.
*******************************************************************************/
/*******************************************************************************
* File Name   : spibsc_init2.c
* $Rev: $
* $Date::                           $
* Description : Flash boot setting
*******************************************************************************/


/******************************************************************************
Includes   <System Includes> , "Project Includes"
******************************************************************************/
#include "spibsc.h"
#include "r_spibsc_ioset_api.h"

#pragma arm section code   = "CODE_SPIBSC_INIT2"
#pragma arm section rodata = "CONST_SPIBSC_INIT2"
#pragma arm section rwdata = "DATA_SPIBSC_INIT2"
#pragma arm section zidata = "BSS_SPIBSC_INIT2"

/******************************************************************************
Typedef definitions
******************************************************************************/


/******************************************************************************
Macro definitions
******************************************************************************/
#define SPIBSC_CH             0
#define DEF_USER_PROGRAM_TOP  0x18080000
#define DEF_SPIBSC_DUAL_MODE  SPIBSC_CMNCR_BSZ_SINGLE

/******************************************************************************
Imported global variables and functions (from other files)
******************************************************************************/
extern void UserProgJmp(uint32_t addr);

/******************************************************************************
Exported global variables and functions (to be accessed by other files)
******************************************************************************/
st_spibsc_cfg_t g_spibsc_cfg;


/******************************************************************************
Private global variables and functions
******************************************************************************/
void spibsc_init2(void);
static void sflash_boot_error_down(void);

/******************************************************************************
* Function Name: spibsc_init2
* Description  :
* Arguments    : none
* Return Value : none
******************************************************************************/
void spibsc_init2(void)
{
    /* Wait TEND=1 for setting change in SPIBSC. */
    R_SFLASH_WaitTend(SPIBSC_CH);

    /* ==== Sets the SPIBSC ==== */
    R_SFLASH_Set_Config(SPIBSC_CH, &g_spibsc_cfg);
    if( R_SFLASH_Exmode_Setting(SPIBSC_CH, DEF_SPIBSC_DUAL_MODE, &g_spibsc_cfg) != 0 )
    {
        sflash_boot_error_down();
    }

    UserProgJmp(DEF_USER_PROGRAM_TOP);
}

/******************************************************************************
* Function Name: sflash_boot_error_down
* Description  :
* Arguments    : none
* Return Value : none
******************************************************************************/
static void sflash_boot_error_down(void)
{
    while(1);
}

/* End of File */
