/*******************************************************************************
* DISCLAIMER
* This software is supplied by Renesas Electronics Corporation and is only
* intended for use with Renesas products. No other uses are authorized. This
* software is owned by Renesas Electronics Corporation and is protected under
* all applicable laws, including copyright laws.
* THIS SOFTWARE IS PROVIDED "AS IS" AND RENESAS MAKES NO WARRANTIES REGARDING
* THIS SOFTWARE, WHETHER EXPRESS, IMPLIED OR STATUTORY, INCLUDING BUT NOT
* LIMITED TO WARRANTIES OF MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE
* AND NON-INFRINGEMENT. ALL SUCH WARRANTIES ARE EXPRESSLY DISCLAIMED.
* TO THE MAXIMUM EXTENT PERMITTED NOT PROHIBITED BY LAW, NEITHER RENESAS
* ELECTRONICS CORPORATION NOR ANY OF ITS AFFILIATED COMPANIES SHALL BE LIABLE
* FOR ANY DIRECT, INDIRECT, SPECIAL, INCIDENTAL OR CONSEQUENTIAL DAMAGES FOR
* ANY REASON RELATED TO THIS SOFTWARE, EVEN IF RENESAS OR ITS AFFILIATES HAVE
* BEEN ADVISED OF THE POSSIBILITY OF SUCH DAMAGES.
* Renesas reserves the right, without notice, to make changes to this software
* and to discontinue the availability of this software. By using this software,
* you agree to the additional terms and conditions found by accessing the
* following link:
* http://www.renesas.com/disclaimer
* Copyright (C) 2012 - 2014 Renesas Electronics Corporation. All rights reserved.
*******************************************************************************/
/*******************************************************************************
* File Name    : main.c
* $Rev: 821 $
* $Date:: 2014-04-21 11:40:10 +0900#$
* Device(s)    : RZ/A1H
* Tool-Chain   : DS-5 Ver 5.16
*              : ARM Complier
* OS           : None
* H/W Platform : R7S72100 CPU Board
* Description  : RZ/A1H Sample Program - Main
* Operation    : 
* Limitations  : 
*******************************************************************************/


/******************************************************************************
Includes   <System Includes> , "Project Includes"
******************************************************************************/
#include <stdio.h>
#include "r_typedefs.h"
#include "dev_drv.h"            /* Device Driver common header */
#include "devdrv_ostm.h"        /* OSTM Driver Header */
#include "main.h"
#include "sample_main.h"
#include "led.h"

#include "iodefine.h"
#include "rza_io_regrw.h"

#include "devdrv_scif_uart.h"
#include "sio_char.h"

/******************************************************************************
Typedef definitions
******************************************************************************/


/******************************************************************************
Macro definitions
******************************************************************************/
#define MAIN_LED_ON     (1)
#define MAIN_LED_OFF    (0)

/******************************************************************************
Imported global variables and functions (from other files)
******************************************************************************/


/******************************************************************************
Exported global variables and functions (to be accessed by other files)
******************************************************************************/


/******************************************************************************
Private global variables and functions
******************************************************************************/
static uint32_t main_led_flg;      /* LED lighting/turning off */
#if defined(__ICCARM__) || defined(__GNUC__)
void SystemInit(void);
#endif


/******************************************************************************
* Function Name: main
* Description  : Displays the sample program information on the terminal 
*              : connected with the CPU board by the UART, and executes initial 
*              : setting for the PORT connected with the LEDs on the board. 
*              : Executes initial setting for the OSTM channel 0, and sets the 
*              : timer counter so that the OSTM0 interrupt occurs every 500ms.
* Arguments    : none
* Return Value : 0
******************************************************************************/
int_t main(void)
{
	volatile unsigned long x;
	unsigned char cc;


#if defined(__ICCARM__) || defined(__GNUC__)
    SystemInit();
#endif

    /* ==== Setup the I/O without buffering ==== */
#ifdef __ICCARM__
    setvbuf( stdout, NULL, _IONBF, 0 );
    setvbuf( stdin,  NULL, _IONBF, 0 );
#endif


    printf("\nRZ/A1L CPU Board S-Flash Sample Program. Ver.1.00\n");
    printf("Copyright (C) 2012 - 2014 Renesas Electronics Corporation. All rights reserved.\n");
    printf("Copyright (C) 2015 Hokuto Denshi Co., Ltd. All rights reserved.\n");
    printf("\n");
    printf("HSBRZA1L176 boot demo.\n");
    printf("SIO3 echo back program.\n");
    printf("\n");
    printf(">");



    /* ==== LED initial setting ==== */
    //LedInit();                      /* PORTfunction setting       */
    //main_led_flg = MAIN_LED_OFF;    /* Initialize LED on/off flag */

    /* ==== Initialize OS timer (channel 0) ==== */
    //R_OSTM_Init(DEVDRV_CH_0, OSTM_MODE_INTERVAL, 500);

    /* ==== Start OS timer (channel 0) ==== */
    //R_OSTM_Open(DEVDRV_CH_0);

    /* ==== Receive command, activate sample software ==== */
    //Sample_Main();


    //SIO3 UART echo back (SIO3 115200bps)
    while(1)
    {
       	cc=IoGetchar3();
       	if(cc == '\r')
       	{
       		IoPutchar3('\r');
       		IoPutchar3('\n');
       		printf(">");
       	}
       	else
       	{
       		IoPutchar3(cc);
       	}
    }


    return 0;
}

/******************************************************************************
* Function Name: Sample_OSTM0_Interrupt
* Description  : This function is executed when the OSTM0 interrupt is received. 
*              : In this sample code, the processing to blink the LEDs on the 
*              : CPU board every 500ms is executed.
* Arguments    : uint32_t int_sense : Interrupt detection
*              :                    :   INTC_LEVEL_SENSITIVE : Level sense
*              :                    :   INTC_EDGE_TRIGGER    : Edge trigger
* Return Value : none
******************************************************************************/
void Sample_OSTM0_Interrupt(uint32_t int_sense)
{
    /* ==== LED blink ==== */
    main_led_flg ^= 1;

    if (MAIN_LED_ON == main_led_flg)
    {
        LedOn();
    }
    else
    {
        LedOff();
    }
}

/* End of File */

